library(shinyWidgets)
library(pdftools)
library(tm)
library(wordcloud2)

books <<- list.files(pattern = "pdf$")
booknames <- c("Autobiographie", "Philosophie et principes mécaniques",
               "Philosophie", "Recherche et pratique")

getTermMatrix <- function(book, frq, max) {
  
  myCorpus <- Corpus(URISource(books), readerControl = list(reader = readPDF))
  myCorpus.tdm <- TermDocumentMatrix(myCorpus, control = list(removePunctuation = T,
                                                              stopwords = T,
                                                              stemming = F,
                                                              removeNumbers = T,
                                                              bounds = list(global = c(3, Inf))))
  
  frqt <- findFreqTerms(myCorpus.tdm, lowfreq = frq)
  mat.words <- data.frame(as.matrix(myCorpus.tdm[frqt,]))
  mat.words <- subset(mat.words, nchar(rownames(mat.words)) > 4)
  
  df <- NULL
  df$root <- gsub("[aeyuio]+$","",stemDocument(rownames(mat.words)))
  df$root <- stemCompletion(df$root,rownames(mat.words),"prevalent")
  mat.words <- aggregate(mat.words, by = list(df$root), FUN = sum)
  
  rownames(mat.words) <- mat.words$Group.1
  mat.words <- mat.words[, !grepl("Group.1", names(mat.words))]
  
  res <- NULL
  if(length(book) > 1) {
    res$word <- rownames(mat.words[, names(mat.words) %in% book])
    res$freq <- rowSums(mat.words[, names(mat.words) %in% book])
  } else {
    res$word <- rownames(mat.words)
    res$freq <- mat.words[, book]
  }
  
  res <- data.frame(res)
  res <- res[res$freq >= frq,]
  res <- res[order(res$freq, decreasing = T),]
  
  if(max > nrow(res)){
    max <- nrow(res)
  }
  
  res <- res[1:max,]
  
  return(res)
}


ui <- fluidPage(
  
  tags$head(
    tags$style(HTML("
            code {
                display:block;
                padding:9.5px;
                margin:0 0 10px;
                margin-top:10px;
                font-size:13px;
                line-height:20px;
                word-break:normal;
                word-wrap:break-word;
                white-space:pre-wrap;
                background-color:#F5F5F5;
                border:1px solid rgba(0,0,0,0.15);
                border-radius:4px; 
                font-family:arial;
                color:#000;
            }"))),

  titlePanel("Words of Osteopathy"),
  
  tabsetPanel(
    tabPanel(
      title = "Nuage",
  
  sidebarLayout(
    
    sidebarPanel(
      checkboxGroupInput("selection",
                         label = "Choisissez un ou plusieurs livres",
                         choiceValues = books,
                         choiceNames = booknames),
      actionButton("update", "Nuage"),
      hr(),
      sliderInput("freq",
                  "Fréquence minimale",
                  min = 50,  max = 200, value = 50),
      sliderInput("max",
                  "Nombre maximal de mots à afficher",
                  min = 20,  max = 100,  value = 100),
      switchInput(inputId = "txtcolor",
                  label = "Couleur du texte",
                  value = T,
                  onLabel = "Sombre", offLabel = "Lumineux")
    ),
    
    mainPanel(
      wordcloud2Output("wordcloud2", width = "100%"),
      br(),
      code("Attention, la taille disponible pour l'affichage impacte directement le nombre de mots qui pourront s'afficher : certains mots très fréquents peuvent ne pas s'afficher s'ils n'en ont pas la place, privilégiez le plein écran.")
    )
  )
  ),
  
  tabPanel(
    title = "A propos",
    
    br(),
    "Cette petite application est basée sur les PDF des 4 ouvrages obtenus sur archive.org :",
    br(),
    HTML("<ul><li> <a href = 'https://archive.org/details/autobiographyofa00stiliala'> Autobiographie </a> </li>
         <li> <a href = 'https://archive.org/details/philosophymecha00stil/mode/2up'> Philosophie et principes mécaniques de l'Ostéopathie </a> </li>
         <li> <a href = 'https://archive.org/details/philosophyofoste00stil'> Philosophie de l'Ostéopathie </a> </li>
         <li> <a href = 'https://archive.org/details/osteopathyresea00stilgoog/page/30/mode/2up'> Recherche et Pratique </a> </li></ul>"),
   br(),
   HTML("Vous pouvez retrouver le code source de l'application sur <a href = 'https://gitlab.univ-nantes.fr/E134497S/wordsofosteopathy/'> Gitlab </a>
        ainsi qu'une sauvegarde des PDFs en question")
    
)
)
)


server <- function(input, output, session) {

  terms <- reactive({
    validate(
      need(!is.null(input$selection), "Sélectionnez un ou plusieurs ouvrages"))
    withProgress({
      setProgress(message = "Nuage en cours...")
      getTermMatrix(input$selection, input$freq, input$max)
      })
    })
  
  output$wordcloud2 <- renderWordcloud2({
    input$update
    isolate({
      v <- terms()
      txtcol <- ifelse(input$txtcolor, "random-dark", "random-light")
      wordcloud2(v, color = txtcol, backgroundColor="transparent")
      })
  })

}
  

shinyApp(ui, server)
